#include "stdafx.h"
#include "DirectShowUtil.h"

#ifdef USE_MEDIA_FOUNDATION
#pragma comment(lib,"mf.lib")
#pragma comment(lib,"mfplat.lib")
#pragma comment(lib,"mfuuid.lib")
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif




CDirectShowFilterFinder::CDirectShowFilterFinder()
{
	::CoInitialize(NULL);
}


CDirectShowFilterFinder::~CDirectShowFilterFinder()
{
	::CoUninitialize();
}


void CDirectShowFilterFinder::Clear()
{
	m_FilterList.clear();
}


int CDirectShowFilterFinder::GetFilterCount()
{
	return m_FilterList.size();
}


bool CDirectShowFilterFinder::GetFilterInfo(const int iIndex,CLSID *pidClass,LPWSTR pwszFriendryName,int iBufLen)
{
	if (iIndex<0 || iIndex>=GetFilterCount())
		return false;
	CFilterInfo &Info = m_FilterList[iIndex];
	if (pidClass)
		*pidClass = Info.m_clsid;
	if (pwszFriendryName) {
		if (Info.m_pwszFriendryName) {
			::lstrcpynW(pwszFriendryName,Info.m_pwszFriendryName,iBufLen);
		} else if (iBufLen>0) {
			pwszFriendryName[0]='\0';
		}
	}
	return true;
}


bool CDirectShowFilterFinder::FindFilter(const CLSID *pidInType,const CLSID *pidInSubType,const CLSID *pidOutType,const CLSID *pidOutSubType)
{
	// idType idSubType ̃tB^(ϊtB^)
	bool bRet = false;
	IFilterMapper2 *pMapper=NULL;
	HRESULT hr=CoCreateInstance(CLSID_FilterMapper2,NULL, CLSCTX_INPROC, IID_IFilterMapper2,(void **) &pMapper);

	if(SUCCEEDED(hr)) {
		IEnumMoniker *pEnum=NULL;
		GUID arInType[2],arOutType[2];
		GUID *pInTypes,*pOutTypes;
		if(pidInType)    arInType[0] = *pidInType;
		if(pidInSubType) arInType[1] = *pidInSubType;
		pInTypes = (pidInType && pidInSubType) ? arInType : NULL;
		if(pidOutType)   arOutType[0] = *pidOutType;
		if(pidOutSubType)arOutType[1] = *pidOutSubType;
		pOutTypes = (pidOutType && pidOutSubType) ? arOutType : NULL;

		hr = pMapper->EnumMatchingFilters(
			&pEnum,
			0,					// \ς
			TRUE,				// Svgp邩
			MERIT_DO_NOT_USE+1,	// ŏ̃bg
			TRUE,				// 1 ȏ̓̓s?
			pInTypes?1:0,		// ͂̃W[^Cv/Tu^Cv̑΂̐
			pInTypes,			// ͂̃W[^Cv/Tu^Cv̑΂̔z
			NULL,				// ̓fBA
			NULL,				// ̓s̃JeS
			FALSE,				// _łȂ΂ȂȂ
			TRUE,				// 1 ȏ̏o̓s
			pOutTypes?1:0,		// o͂̃W[^Cv/Tu^Cv̑΂̐
			pOutTypes,			// o͂̃W[^Cv/Tu^Cv̑΂̔z
			NULL,				// o̓fBA
			NULL);				// o̓s̃JeS
		if(SUCCEEDED(hr)){
			IMoniker *pMoniker;
			ULONG cFetched;
			while(pEnum->Next(1, &pMoniker, &cFetched) == S_OK){
				IPropertyBag *pPropBag;
				hr=pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);
				if(SUCCEEDED(hr)){
					VARIANT varName,varID;
					VariantInit(&varName);
					VariantInit(&varID);
					hr=pPropBag->Read(L"FriendlyName", &varName, 0);
					if(SUCCEEDED(hr)){
						hr=pPropBag->Read(L"CLSID", &varID, 0);
						if(SUCCEEDED(hr)){
							bRet = true;
							CFilterInfo FilterInfo;
							FilterInfo.SetFriendryName(varName.bstrVal);
							CLSIDFromString(varID.bstrVal,&FilterInfo.m_clsid);
							m_FilterList.push_back(FilterInfo);
							SysFreeString(varID.bstrVal);
							}
							SysFreeString(varName.bstrVal);
						}
						pPropBag->Release();
					}
				}
				pEnum->Release();
			}
			pMapper->Release();
		}
	return bRet;
}


// D悷tB^Xg[ɎĂ
bool CDirectShowFilterFinder::PriorityFilterGoToHead(const CLSID idPriorityClass)
{
	std::vector<CFilterInfo> TmpList;
	size_t i;

	for (i=0;i<m_FilterList.size();i++) {
		if (m_FilterList[i].m_clsid == idPriorityClass) {
			// D悷̂𔭌
			TmpList.push_back(m_FilterList[i]);
		}
	}
	if (!TmpList.empty()) {
		for (i=0;i<m_FilterList.size();i++) {
			if(m_FilterList[i].m_clsid != idPriorityClass) {
				// D悷̈ȊO
				TmpList.push_back(m_FilterList[i]);
			}
		}
	}
	m_FilterList=TmpList;
	return true;
}


// tB^XgI[ɎĂ
bool CDirectShowFilterFinder::IgnoreFilterGoToTail(const CLSID idIgnoreClass,bool bRemoveIt)
{
	std::vector<CFilterInfo> TmpList;
	size_t i;

	for (i=0;i<m_FilterList.size();i++) {
		if (m_FilterList[i].m_clsid != idIgnoreClass) {
			// ̈ȊO
			TmpList.push_back(m_FilterList[i]);
		}
	}
	if (!bRemoveIt) {
		for (i=0;i<m_FilterList.size();i++) {
			if (m_FilterList[i].m_clsid == idIgnoreClass) {
				TmpList.push_back(m_FilterList[i]);
			}
		}
	}
	m_FilterList=TmpList;
	return true;
}




CDirectShowFilterFinder::CFilterInfo::CFilterInfo()
	: m_pwszFriendryName(NULL)
{
}


CDirectShowFilterFinder::CFilterInfo::CFilterInfo(const CFilterInfo &Info)
	: m_pwszFriendryName(NULL)
{
	*this=Info;
}


CDirectShowFilterFinder::CFilterInfo::~CFilterInfo()
{
	delete m_pwszFriendryName;
}


CDirectShowFilterFinder::CFilterInfo &CDirectShowFilterFinder::CFilterInfo::operator=(const CFilterInfo &Info)
{
	SetFriendryName(Info.m_pwszFriendryName);
	m_clsid=Info.m_clsid;
	return *this;
}


void CDirectShowFilterFinder::CFilterInfo::SetFriendryName(LPCWSTR pwszFriendryName)
{
	if (m_pwszFriendryName) {
		delete [] m_pwszFriendryName;
		m_pwszFriendryName=NULL;
	}
	if (pwszFriendryName) {
		m_pwszFriendryName=new WCHAR[::lstrlenW(pwszFriendryName)+1];
		::lstrcpyW(m_pwszFriendryName,pwszFriendryName);
	}
}




HRESULT DirectShowUtil::AddToRot(IUnknown *pUnkGraph, DWORD *pdwRegister)
{
	IMoniker * pMoniker;
	IRunningObjectTable *pROT;

	if (FAILED(::GetRunningObjectTable(0, &pROT)))
		return E_FAIL;

	wchar_t wsz[256];
	swprintf_s(wsz,256, L"FilterGraph %08p pid %08x", (DWORD_PTR)pUnkGraph, ::GetCurrentProcessId());

	HRESULT hr = ::CreateItemMoniker(L"!", wsz, &pMoniker);

	if (SUCCEEDED(hr)) {
		hr = pROT->Register(0, pUnkGraph, pMoniker, pdwRegister);
		pMoniker->Release();
	}
	pROT->Release();
	return hr;
}


void DirectShowUtil::RemoveFromRot(const DWORD dwRegister)
{
	IRunningObjectTable *pROT;

	if (SUCCEEDED(::GetRunningObjectTable(0, &pROT))) {
		pROT->Revoke(dwRegister);
		pROT->Release();
	}
}


// tB^w胁fBÃs
IPin* DirectShowUtil::GetFilterPin(IBaseFilter *pFilter, const PIN_DIRECTION dir, const AM_MEDIA_TYPE *pMediaType)
{
	IEnumPins *pEnumPins=NULL;
	IPin *pPin;
	IPin *pRetPin=NULL;

	if(pFilter->EnumPins(&pEnumPins)==S_OK ){
		ULONG cFetched;
		while(!pRetPin&&pEnumPins->Next(1,&pPin,&cFetched)==S_OK){
			PIN_INFO stPin;
			if(pPin->QueryPinInfo(&stPin)==S_OK){
				if(stPin.dir==dir){
					if(!pMediaType){
						// ĂOK
						pRetPin=pPin;
						pRetPin->AddRef();
						} else {
						// DirectShowɂ܂ăs
						if(pPin->QueryAccept(pMediaType)==S_OK){
							pRetPin=pPin;
							pRetPin->AddRef();
							}
						}
					}
					if(stPin.pFilter) stPin.pFilter->Release();
				}
				pPin->Release();
			}
		pEnumPins->Release();
		}
	return pRetPin;
}

// tB^̃vpeBy[WJ
bool DirectShowUtil::ShowPropertyPage(IBaseFilter *pFilter, HWND hWndParent)
{
	if (!pFilter)
		return false;

	bool bRet = false;
	ISpecifyPropertyPages *pProp = NULL;

	HRESULT hr = pFilter->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pProp);
	if(SUCCEEDED(hr)){
		CAUUID caGUID;
		FILTER_INFO stFilterInfo;

		pProp->GetPages(&caGUID);
		SAFE_RELEASE(pProp);
		hr = pFilter->QueryFilterInfo(&stFilterInfo);
		if(SUCCEEDED(hr)){
			IUnknown *pFilterUnk=NULL;

			hr = pFilter->QueryInterface(IID_IUnknown, (void **)&pFilterUnk);
			if(SUCCEEDED(hr)){
				OleCreatePropertyFrame(
					hWndParent,             // eEBhEB
					0, 0,                   // \ς݁B
					stFilterInfo.achName,   // _CAO {bNX̃LvVB
					1,                      // IuWFNg (tB^̂)B
					&pFilterUnk,            // IuWFNg |C^̔zB
					caGUID.cElems,          // vpeB y[WB
					caGUID.pElems,          // vpeB y[W CLSID ̔zB
					0,                      // P[ʎqB
					0, NULL                 // \ς݁B
				);
				CoTaskMemFree(caGUID.pElems);
				SAFE_RELEASE(pFilterUnk);
				bRet = true;
			}
			SAFE_RELEASE(stFilterInfo.pGraph);
		}
	}
	return bRet;
}


#if 0
// MpegfR[_ǉăsڑsB
// pUtilɐς݃fR[_񋓂ĂƂ̐񏇂ɐڑsBNULLȂftHgB
//   킴킴w肷闝R : ̃NXɌꂽtB^Mpeg2̂̂Ƃ͌ȂׁB
//
// ppMpeg2DecoderFilter ͐ڑɎgꂽMpeg2C^[tF[X󂯂ƂB
// ppCurrentOutputPin 㗬ɐڑꂽLȃsł̂OB
// ppCurrentOutputPin ͐IȂAfR[_tB^̏o̓s*ppNewOutputPinɂȂB
// ppNewOutputPin==NULL ̏ꍇAfR[_tB^̏o̓s*ppCurrentOutputPinɂȂB*ppCurrentOutputPin͉
//
// sꍇłtB^͍sȂpX(ɐڑς݂̏ꍇ)B߂tB^͊mFĉ邱
bool DirectShowUtil::AppendMpeg2Decoder_and_Connect(IGraphBuilder *pFilterGraph, CDirectShowFilterFinder *pUtil, IBaseFilter **ppMpeg2DecoderFilter,wchar_t *lpszDecoderName,int iDecNameBufLen, IPin **ppCurrentOutputPin, IPin **ppNewOutputPin)
{
	HRESULT hr;
	IPin *pInput = NULL;
	CDirectShowFilterFinder cUtil;	
	CLSID guidFilter = CLSID_NULL;
	bool bRet;
	wchar_t tmp[128];
	if(!lpszDecoderName){
		lpszDecoderName = tmp;
		iDecNameBufLen = 128;
		}

	// |C^`FbN
	if(!pFilterGraph || !ppMpeg2DecoderFilter || !ppCurrentOutputPin) return false;
	// sAhXȂ New==NULL œ͂ꂽ̂Ɠ`
	if(ppCurrentOutputPin==ppNewOutputPin) ppNewOutputPin = NULL;
	// w肪Ȃꍇ̃tB^
	if(!pUtil){
		pUtil = &cUtil;
		if(!pUtil->FindFilter(&MEDIATYPE_Video,&MEDIASUBTYPE_MPEG2_VIDEO)) return false;
		}
	// ߂lNA
	*ppMpeg2DecoderFilter = NULL;

	bRet=false;
	for(int i=0;!bRet&&i<pUtil->GetFilterCount();i++){
		if(!pUtil->GetFilterInfo(i,&guidFilter,lpszDecoderName,iDecNameBufLen)) continue;
		hr = ::CoCreateInstance(guidFilter, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void**)ppMpeg2DecoderFilter);
		if(FAILED(hr)) continue;
		hr = pFilterGraph->AddFilter(*ppMpeg2DecoderFilter,lpszDecoderName);
		if(FAILED(hr)){
			SAFE_RELEASE(*ppMpeg2DecoderFilter);
			continue;
			}
		pInput = CDirectShowFilterFinder::GetFilterPin(*ppMpeg2DecoderFilter,PINDIR_INPUT);
		if(!pInput){
			SAFE_RELEASE(*ppMpeg2DecoderFilter);
			continue;
		}
		hr = pFilterGraph->Connect(*ppCurrentOutputPin,pInput);
		if(FAILED(hr)){
			SAFE_RELEASE(pInput);
			pFilterGraph->RemoveFilter(*ppMpeg2DecoderFilter);			
			SAFE_RELEASE(*ppMpeg2DecoderFilter);
			continue;
		} else {
			bRet=true;
		}
		}
	if(!bRet) {
		// Sgݍ킹œKfR[_
		return false;
	}
	// ڑɎgs
	SAFE_RELEASE(*ppCurrentOutputPin);
	SAFE_RELEASE(pInput);
	// tB^ւ̏o̓sT
	IPin *pNewOutput = GetFilterPin(*ppMpeg2DecoderFilter, PINDIR_OUTPUT);
	if(!pNewOutput){
		// o̓sȂ
		return false;
		}
	if(ppNewOutputPin){
		// Vo̓sɏo
		// ̏o̓s͊ɉς
		*ppNewOutputPin = pNewOutput;
		}else{
		// o̓sXV(ppCurrentOutputPin==ppNewOutputPin̏ꍇł̂㏑XVƂȂ)
		// ̏o̓s͊ɉς
		*ppCurrentOutputPin = pNewOutput;
		}
	return true;
}
#endif

// wtB^oRăsڑs(ɓ=1/o=1̌oR^tB^ڑp)
// Filterw
//
// lpwszFilterName  NULL łǂB
// ppCurrentOutputPin 㗬ɐڑꂽLȃsł̂OB
// ppCurrentOutputPin ͐IȂAtB^̏o̓s*ppNewOutputPinɂȂB
// ppNewOutputPin==NULL ̏ꍇAtB^̏o̓s*ppCurrentOutputPinɂȂB*ppCurrentOutputPin͉
//
bool DirectShowUtil::AppendFilterAndConnect(IGraphBuilder *pFilterGraph,
				IBaseFilter *pFilter,LPCWSTR lpwszFilterName,
				IPin **ppCurrentOutputPin,IPin **ppNewOutputPin,bool fDirect)
{
	HRESULT hr;

	// |C^`FbN
	if (!pFilterGraph || !pFilter || !ppCurrentOutputPin)
		return false;
	// sAhXȂ New==NULL œ͂ꂽ̂Ɠ`
	if (ppCurrentOutputPin==ppNewOutputPin)
		ppNewOutputPin = NULL;
	if (!lpwszFilterName)
		lpwszFilterName = L"No Name";
	hr = pFilterGraph->AddFilter(pFilter, lpwszFilterName);
	if (FAILED(hr)) {
		// tB^ɒǉs
		return false;
	}
	IPin *pInput = GetFilterPin(pFilter, PINDIR_INPUT);
	if (!pInput) {
		// ̓sȂ
		pFilterGraph->RemoveFilter(pFilter);
		return false;
	}
	// ڑ
	if (fDirect)
		hr = pFilterGraph->ConnectDirect(*ppCurrentOutputPin,pInput,NULL);
	else
		hr = pFilterGraph->Connect(*ppCurrentOutputPin,pInput);
	if (FAILED(hr)) {
		// ڑłȂ
		SAFE_RELEASE(pInput);
		pFilterGraph->RemoveFilter(pFilter);
		return false;
	}
	// ڑɎgs
	SAFE_RELEASE(*ppCurrentOutputPin);
	SAFE_RELEASE(pInput);
	// tB^ւ̏o̓sT(o̓sĂ͑s)
	IPin *pNewOutput = GetFilterPin(pFilter, PINDIR_OUTPUT);
	if (ppNewOutputPin) {
		// Vo̓sɏo
		// ̏o̓s͊ɉς
		*ppNewOutputPin = pNewOutput;
	} else {
		// o̓sXV(ppCurrentOutputPin==ppNewOutputPin̏ꍇł̂㏑XVƂȂ)
		// ̏o̓s͊ɉς
		*ppCurrentOutputPin = pNewOutput;
	}
#ifdef _DEBUG
	LONG refCount = GetRefCount(pFilter);
#endif
	return true;
}

// wtB^oRăsڑs(ɓ=1/o=1̌oR^tB^ڑp)
// CLSIDw
//
// AppendFilterAndConnect(Filterw)̐QƁB
// guidFilter͗LDirectShowtB^GUIDw肷B
// ppAppendedFilter ͒ǉtB^󂯎B
bool DirectShowUtil::AppendFilterAndConnect(IGraphBuilder *pFilterGraph,
	const CLSID guidFilter,LPCWSTR lpwszFilterName,IBaseFilter **ppAppendedFilter,
	IPin **ppCurrentOutputPin,IPin **ppNewOutputPin,bool fDirect)
{
	// tB^CX^X쐬
	HRESULT hr = ::CoCreateInstance(guidFilter, NULL, CLSCTX_INPROC_SERVER,
				IID_IBaseFilter, reinterpret_cast<LPVOID*>(ppAppendedFilter));
	if (FAILED(hr)) {
		// CX^X쐬s
		return false;
	}
	bool bRet = AppendFilterAndConnect(pFilterGraph,*ppAppendedFilter,lpwszFilterName,
									ppCurrentOutputPin,ppNewOutputPin,fDirect);
	if (!bRet) {
		SAFE_RELEASE(*ppAppendedFilter);
		return false;
	}
	return true;
}

// FԕϊtB^oRăsڑs(GtFNgtB^ւ̑Ή̂߂̐FԕϊKvȏꍇ̂)
//
// AppendFilterAndConnect(Filterw)̐Q
// ppColorSpaceConverterFilter ͒ǉtB^󂯎ׂ̃|C^
bool DirectShowUtil::AppendColorSpaceConverterFilter_and_Connect(IGraphBuilder *pFilterGraph, IBaseFilter **ppColorSpaceConverterFilter, IPin **ppCurrentOutputPin, IPin **ppNewOutputPin)
{
	return AppendFilterAndConnect(pFilterGraph,CLSID_ColorSpaceConverter,L"Color Space Converter",ppColorSpaceConverterFilter,ppCurrentOutputPin,ppNewOutputPin);
}

// _̏o͐rfIEBhẼC^[tF[X擾
// EVRgpMF_GetVideoDisplayControl()g
IVideoWindow* DirectShowUtil::GetVideoWindow(IGraphBuilder *pGraph)
{
	IVideoWindow *pVideoWindow;
	HRESULT hr = pGraph->QueryInterface(IID_IVideoWindow,(void**)&pVideoWindow);
	if(SUCCEEDED(hr)){
		return pVideoWindow;
		}
	return NULL;
}

IBasicVideo2* DirectShowUtil::GetBasicVideo2(IGraphBuilder *pGraph)
{
	IBasicVideo2 *pBasicVideo;
	HRESULT hr = pGraph->QueryInterface(IID_IBasicVideo,(void**)&pBasicVideo);
	if(SUCCEEDED(hr)){
		return pBasicVideo;
		}
	return NULL;
}

IMediaControl* DirectShowUtil::GetMediaControl(IGraphBuilder *pGraph)
{
	IMediaControl *pMediaControl;
	HRESULT hr = pGraph->QueryInterface(IID_IMediaControl,(void**)&pMediaControl);
	if(SUCCEEDED(hr)){
		return pMediaControl;
		}
	return NULL;
}

bool DirectShowUtil::FilterGrapph_Play(IGraphBuilder *pFilterGraph)
{
	bool bRet=false;

	if(pFilterGraph){
		IMediaControl *pControl = GetMediaControl(pFilterGraph);
		if(pControl){
			HRESULT hr = pControl->Run();
			SAFE_RELEASE(pControl);
			bRet = true;
		}
	}
	return bRet;
}

bool DirectShowUtil::FilterGrapph_Stop(IGraphBuilder *pFilterGraph)
{
	bool bRet=false;

	if(pFilterGraph){
		IMediaControl *pControl = GetMediaControl(pFilterGraph);
		if(pControl){
			HRESULT hr = pControl->Stop();
			SAFE_RELEASE(pControl);
			bRet = true;
		}
	}
	return bRet;
}

bool DirectShowUtil::FilterGrapph_Pause(IGraphBuilder *pFilterGraph)
{
	bool bRet=false;

	if(pFilterGraph){
		IMediaControl *pControl = GetMediaControl(pFilterGraph);
		if(pControl){
			HRESULT hr = pControl->Pause();
			SAFE_RELEASE(pControl);
			bRet = true;
		}
	}
	return bRet;
}


//////////////////////////////////////////////////////////////////////
// ȉ EVRp[eBeB

#ifdef USE_MEDIA_FOUNDATION

// EVRp : 
void DirectShowUtil::MF_Init()
{
	::MFStartup(MF_VERSION);
};

// EVRp : I
void DirectShowUtil::MF_Term() {
	::MFShutdown();
};

// EVRp : EVRݒpC^[tF[X̎擾
IEVRFilterConfig* DirectShowUtil::MF_GetEVRFilterConfig(IBaseFilter *pEvr)
{
	IEVRFilterConfig *pEvrFilterConfig;
	HRESULT hr = pEvr->QueryInterface(IID_IEVRFilterConfig,(void**)&pEvrFilterConfig);
	if(SUCCEEDED(hr)) {
		return pEvrFilterConfig;
		}
	return NULL;
}

// EVRp : EVRMF̃T[rX擾pC^[tF[X̎擾
IMFGetService* DirectShowUtil::MF_GetService(IBaseFilter *pEvr)
{
	IMFGetService *pMFGetService;
	HRESULT hr = pEvr->QueryInterface(IID_IMFGetService,(void**)&pMFGetService);
	if(SUCCEEDED(hr)) {
		return pMFGetService;
		}
	return NULL;
}

// EVRp : ̓Xg[ݒ肷iʏ͂Pj
bool DirectShowUtil::MF_SetNumberOfStreams(IBaseFilter *pEvr,int iStreamNumber)
{
	IEVRFilterConfig *pFilterConfig = MF_GetEVRFilterConfig(pEvr);
	if(pFilterConfig) {
		pFilterConfig->SetNumberOfStreams(iStreamNumber);
		SAFE_RELEASE(pFilterConfig);
		return true;
		}
	return false;
}

// EVRp : fBXvCpC^[tF[X̎擾
IMFVideoDisplayControl* DirectShowUtil::MF_GetVideoDisplayControl(IBaseFilter *pEvr)
{
	IMFGetService *pService = MF_GetService(pEvr);
	if(pService) {
		IMFVideoDisplayControl *pDisplayControl;
		HRESULT hr = pService->GetService(MR_VIDEO_RENDER_SERVICE,IID_IMFVideoDisplayControl,(void**)&pDisplayControl);
		SAFE_RELEASE(pService);
		if(SUCCEEDED(hr)){
			return pDisplayControl;
			}
		}
	return NULL;
}

// EVRp : rfI~LTpC^[tF[X̎擾
IMFVideoMixerControl* DirectShowUtil::MF_GetVideoMixerControl(IBaseFilter *pEvr)
{
	IMFGetService *pService = MF_GetService(pEvr);
	if(pService) {
		IMFVideoMixerControl *pVideoMixerControl;
		HRESULT hr = pService->GetService(MR_VIDEO_MIXER_SERVICE,IID_IMFVideoMixerControl,(void**)&pVideoMixerControl);
		SAFE_RELEASE(pService);
		if(SUCCEEDED(hr)) {
			return pVideoMixerControl;
			}
		}
	return NULL;
}

// EVRp : rfIvZbTpC^[tF[X̎擾
IMFVideoProcessor* DirectShowUtil::MF_GetVideoProcessor(IBaseFilter *pEvr)
{
	IMFGetService *pService = MF_GetService(pEvr);
	if(pService) {
		IMFVideoProcessor *pVideoProcessor;
		HRESULT hr = pService->GetService(MR_VIDEO_MIXER_SERVICE,IID_IMFVideoProcessor,(void**)&pVideoProcessor);
		SAFE_RELEASE(pService);
		if(SUCCEEDED(hr)) {
			return pVideoProcessor;
			}
		}
	return NULL;
}

#endif
